@@ -23,214 +23,243 @@
 */
 
 #include "audiophash.h"
+#include <sndfile.h>
+#include <samplerate.h>
+#include "config.h"
+
+#ifdef HAVE_LIBMPG123
+#include <mpg123.h>
+#endif
+
 int ph_count_samples(const char *filename, int sr,int channels){
 
-
-    av_log_set_level(AV_LOG_QUIET);
-	av_register_all();
-	avcodec_register_all();
-
-	AVFormatContext *pFormatCtx;
-	
-	// Open file
-	if(av_open_input_file(&pFormatCtx, filename, NULL, 0, NULL)!=0)
-	  return -1 ; // Couldn't open file
-	 
-	// Retrieve stream information
-	if(av_find_stream_info(pFormatCtx)<0)
-	  return -1; // Couldn't find stream information
-	
-	//dump_format(pFormatCtx,0,NULL,0);//debugging function to print infomation about format
-
-	unsigned int i;
-	AVCodecContext *pCodecCtx;
-
-	// Find the video stream
-	int audioStream=-1;
-
-	for(i=0; i<pFormatCtx->nb_streams; i++)
-	{
-	     if(pFormatCtx->streams[i]->codec->codec_type==CODEC_TYPE_AUDIO) 
-	     {
-		    audioStream=i;
-		    break;
-	     }
-	}
-	if(audioStream==-1){
-	     return -1; //no video stream
-	}
-	
-	// Get a pointer to the codec context for the audio stream
-	pCodecCtx=pFormatCtx->streams[audioStream]->codec;
-
-	int src_sr = pCodecCtx->sample_rate;
-        int src_channels = pCodecCtx->channels;
-
-	AVCodec *pCodec;
-
-	// Find the decoder
-	pCodec=avcodec_find_decoder(pCodecCtx->codec_id);
-	if(pCodec==NULL) 
-	{
-	  	return -1 ; // Codec not found
-	}
-	// Open codec
-	if(avcodec_open(pCodecCtx, pCodec)<0)
-	  return -1; // Could not open codec
-
-	uint8_t *in_buf = (uint8_t *)malloc(AVCODEC_MAX_AUDIO_FRAME_SIZE + FF_INPUT_BUFFER_PADDING_SIZE);
-        uint8_t *out_buf = (uint8_t *)malloc(AVCODEC_MAX_AUDIO_FRAME_SIZE);
-
-        int in_buf_used, numbytesread, buf_size = AVCODEC_MAX_AUDIO_FRAME_SIZE;
-
-
-	ReSampleContext *rs_ctxt = audio_resample_init(channels, src_channels, sr, src_sr);
-
-	int count = 0;
-	AVPacket packet;
-	while(av_read_frame(pFormatCtx, &packet)>=0) 
-	{
-	    in_buf_used = buf_size;
-	    numbytesread = avcodec_decode_audio2(pCodecCtx,(int16_t*)in_buf,&in_buf_used,packet.data,packet.size);  
-	    if (numbytesread <= 0){
-		continue;
-	    }
-	    count += (int)(audio_resample(rs_ctxt,(short*)out_buf ,(short*)in_buf,in_buf_used)/sizeof(int16_t));
-	}
-	free(in_buf);
-	free(out_buf);
-	audio_resample_close(rs_ctxt);
-	avcodec_close(pCodecCtx);
-	av_close_input_file(pFormatCtx);
-	
-	return count;
+    SF_INFO sf_info;
+    sf_info.format=0;
+    SNDFILE *sndfile = sf_open(filename, SFM_READ, &sf_info);
+    if (sndfile == NULL){
+	return NULL;
+    }
+    int count = sf_info.frames;
+    sf_close(sndfile);
+    return count;
 }
 
-float* ph_readaudio(const char *filename, int sr, int channels, float *sigbuf, int &buflen, const float nbsecs)
-{
-        av_log_set_level(AV_LOG_QUIET);
-	av_register_all();
+#ifdef HAVE_LIBMPG123
 
-	AVFormatContext *pFormatCtx;
-	
-	// Open file
-	if(av_open_input_file(&pFormatCtx, filename, NULL, 0, NULL)!=0){
-	    buflen=0;
-	    return NULL ; // Couldn't open file
+static
+float* readaudio_mp3(const char *filename,long *sr, const float nbsecs, unsigned int *buflen){
+  mpg123_handle *m;
+  int ret;
+
+  if (mpg123_init() != MPG123_OK || ((m = mpg123_new(NULL,&ret)) == NULL)|| \
+                         mpg123_open(m, filename) != MPG123_OK){
+    fprintf(stderr,"unable to init mpg\n");
+    return NULL;
+  }
+
+  /*turn off logging */
+  mpg123_param(m, MPG123_ADD_FLAGS, MPG123_QUIET, 0);
+
+  off_t totalsamples;
+  
+  mpg123_scan(m);
+  totalsamples = mpg123_length(m);
+  
+  int meta = mpg123_meta_check(m);
+
+  int channels, encoding;
+    
+  if (mpg123_getformat(m, sr, &channels, &encoding) != MPG123_OK){
+    fprintf(stderr,"unable to get format\n");
+    return NULL;
+  }
+  
+  mpg123_format_none(m);
+  mpg123_format(m, *sr, channels, encoding);
+
+  size_t decbuflen = mpg123_outblock(m);
+  unsigned char *decbuf = (unsigned char*)malloc(decbuflen);  
+  if (decbuf == NULL){
+    printf("mem alloc error\n");
+    return NULL;
+  }
+
+  unsigned int nbsamples = (nbsecs <= 0) ? totalsamples : nbsecs*(*sr);
+  nbsamples = (nbsamples < totalsamples) ? nbsamples : totalsamples;
+
+  size_t i, j, index = 0, done;
+
+
+  float *buffer = (float*)malloc(nbsamples*sizeof(float));
+  *buflen = nbsamples;
+
+  do {
+    
+    ret = mpg123_read(m, decbuf, decbuflen, &done);
+    switch (encoding) {
+    case MPG123_ENC_SIGNED_16 :
+      for (i = 0; i < done/sizeof(short); i+=channels){
+	buffer[index] = 0.0f;
+	for (j = 0; j < channels ; j++){
+	  buffer[index] += (float)(((short*)decbuf)[i+j])/(float)SHRT_MAX;
 	}
-	 
-	// Retrieve stream information
-	if(av_find_stream_info(pFormatCtx)<0){
-	    buflen=0;
-            av_close_input_file(pFormatCtx);
-	    return NULL; // Couldn't find stream information
+	buffer[index++] /= channels;
+	if (index >= nbsamples) break;
+      }
+      break;
+    case MPG123_ENC_SIGNED_8:
+      for (i = 0; i < done/sizeof(char); i+=channels){
+	buffer[index] = 0.0f;
+	for (j = 0; j < channels ; j++){
+	  buffer[index] += (float)(((char*)decbuf)[i+j])/(float)SCHAR_MAX;
 	}
-	
-	//dump_format(pFormatCtx,0,NULL,0);//debugging function to print infomation about format
-
-	unsigned int i;
-	AVCodecContext *pCodecCtx;
-
-	// Find the video stream
-	int audioStream=-1;
-
-	for(i=0; i<pFormatCtx->nb_streams; i++)
-	{
-	     if(pFormatCtx->streams[i]->codec->codec_type==CODEC_TYPE_AUDIO) 
-	     {
-		    audioStream=i;
-		    break;
-	     }
+	buffer[index++] /= channels;
+	if (index >= nbsamples) break;
+      }
+      break;
+    case MPG123_ENC_FLOAT_32:
+      for (i = 0; i < done/sizeof(float); i+=channels){
+	buffer[index] = 0.0f;
+	for (j = 0; j < channels; j++){
+	  buffer[index] += ((float*)decbuf)[i+j];
 	}
-	if(audioStream==-1){
-	     buflen = 0;
-             av_close_input_file(pFormatCtx);
-	     return NULL; //no video stream
-	}
-	
-	// Get a pointer to the codec context for the audio stream
-	pCodecCtx=pFormatCtx->streams[audioStream]->codec;
+	buffer[index++] /= channels;
+	if (index >= nbsamples) break;
+      }
+      break;
+    default:
+	done = 0;
+    }
 
-        SampleFormat src_fmt = SAMPLE_FMT_S16;
-        SampleFormat dst_fmt = SAMPLE_FMT_S16;
-	int src_sr = pCodecCtx->sample_rate;
-        int src_channels = pCodecCtx->channels;
-        int64_t totalduration = pFormatCtx->streams[audioStream]->duration;
-	int64_t duration = (nbsecs > 0.0f) ? (int64_t)(sr*nbsecs) : totalduration;
-        duration = (duration <= totalduration) ? duration : totalduration;
-        float *buf = NULL;
-        if (!sigbuf || buflen <= duration){
-	    buf = (float*)malloc(duration*sizeof(float)); /* alloc new buffer */ 
-            buflen = duration;
-	} else {
-	    buf = sigbuf; /* use buffer handed in param */ 
-	}
-	AVCodec *pCodec;
+  } while (ret == MPG123_OK && index < nbsamples);
 
-	// Find the decoder
-	pCodec=avcodec_find_decoder(pCodecCtx->codec_id);
-	if(pCodec==NULL) 
-	{
-	        buflen=0;
-                av_close_input_file(pFormatCtx);
-	  	return NULL ; // Codec not found
-	}
-	// Open codec
-	if(avcodec_open(pCodecCtx, pCodec)<0){
-	    buflen=0;
-            av_close_input_file(pFormatCtx);
-	    return NULL; // Could not open codec
-	}
+  free(decbuf);
+  mpg123_close(m);
+  mpg123_delete(m);
+  mpg123_exit();
 
-	uint8_t *in_buf = (uint8_t*)av_malloc(AVCODEC_MAX_AUDIO_FRAME_SIZE + FF_INPUT_BUFFER_PADDING_SIZE);
-        int in_buf_used, numbytesread, buf_size = AVCODEC_MAX_AUDIO_FRAME_SIZE;
-	uint8_t *out_buf = (uint8_t*)av_malloc(AVCODEC_MAX_AUDIO_FRAME_SIZE);
-        int16_t *out_buf16 = (int16_t*)out_buf;
+  return buffer;
+}
 
-	ReSampleContext *rs_ctx = audio_resample_init(channels, src_channels, sr, src_sr);
-        int64_t index = 0;
-	AVPacket *packet = (AVPacket*)malloc(sizeof(AVPacket));
-        av_init_packet(packet);
-	while(av_read_frame(pFormatCtx, packet)>=0 && index < duration) 
-	{
-            while (packet->size > 0){
-		in_buf_used = buf_size;
-		numbytesread = avcodec_decode_audio2(pCodecCtx,(int16_t*)in_buf,&in_buf_used,packet->data,packet->size);  
-		if (numbytesread < 0){
-                    buflen = 0;
-                    if (buf != sigbuf) free(buf);
-                    buf = NULL;
-		    goto audio_cleanup;
-		}
-                if (in_buf_used > 0){
-		    int cnt = audio_resample(rs_ctx,(short*)out_buf,(short*)in_buf,(int)(in_buf_used/sizeof(int16_t)));
-		    if (index + cnt > duration){
-			goto audio_cleanup;
-		    }   
-		    for (int i=0;i<cnt;i++){
-			buf[index+i] = ((float)out_buf16[i]/(float)SHRT_MAX);
-		    }
-		    index += cnt;
-		}
-		packet->size -= numbytesread;
-                packet->data += numbytesread;
-	    }
-            av_destruct_packet_nofree(packet);
+#endif /*HAVE_LIBMPG123*/
+
+static
+float *readaudio_snd(const char *filename, long *sr, const float nbsecs, unsigned int *buflen){
+
+    SF_INFO sf_info;
+    sf_info.format=0;
+    SNDFILE *sndfile = sf_open(filename, SFM_READ, &sf_info);
+    if (sndfile == NULL){
+      return NULL;
+    }
+    
+    /* normalize */ 
+    sf_command(sndfile, SFC_SET_NORM_FLOAT, NULL, SF_TRUE);
+
+    *sr = (long)sf_info.samplerate;
+
+    //allocate input buffer for signal
+    unsigned int src_frames = (nbsecs <= 0) ? sf_info.frames : (nbsecs*sf_info.samplerate);
+    src_frames = (sf_info.frames < src_frames) ? sf_info.frames : src_frames;
+    float *inbuf = (float*)malloc(src_frames*sf_info.channels*sizeof(float));
+
+    /*read frames */ 
+    sf_count_t cnt_frames = sf_readf_float(sndfile, inbuf, src_frames);
+
+    float *buf = (float*)malloc(cnt_frames*sizeof(float));   
+    
+    //average across all channels
+    int  i,j,indx=0;
+    for (i=0;i<cnt_frames*sf_info.channels;i+=sf_info.channels){
+	buf[indx] = 0;
+	for (j=0;j<sf_info.channels;j++){
+	    buf[indx] += inbuf[i+j];
 	}
+	buf[indx++] /= sf_info.channels;
+    }
+    free(inbuf);
 
- audio_cleanup:
-        free(packet);
-	av_free(in_buf);
-	av_free(out_buf);
-	audio_resample_close(rs_ctx);
-	avcodec_close(pCodecCtx);
-	av_close_input_file(pFormatCtx);
-        buflen = (int)index;
-	
-	return buf;
+    *buflen = indx;
+    return buf;
+}
+
+float* ph_readaudio2(const char *filename, int sr, float *sigbuf, int &buflen, const float nbsecs){
+
+  long orig_sr;
+  float *inbuffer = NULL;
+  unsigned int inbufferlength;
+  buflen = 0;
+
+  const char *suffix = strrchr(filename, '.');
+  if (suffix == NULL) return NULL;
+  if (!strcasecmp(suffix+1, "mp3")) {
+#ifdef HAVE_LIBMPG123
+    inbuffer = readaudio_mp3(filename, &orig_sr, nbsecs, &inbufferlength);
+#endif /* HAVE_LIBMPG123 */
+  } else {
+    inbuffer = readaudio_snd(filename, &orig_sr, nbsecs, &inbufferlength);
+  }  
+
+  if (inbuffer == NULL){
+    return NULL;
+  }
+
+  /* resample float array */ 
+  /* set desired sr ratio */ 
+  double sr_ratio = (double)(sr)/(double)orig_sr;
+  if (src_is_valid_ratio(sr_ratio) == 0){
+    free(inbuffer);
+    return NULL;
+  }
+
+  /* allocate output buffer for conversion */ 
+  unsigned int outbufferlength = sr_ratio*inbufferlength;
+  float *outbuffer = (float*)malloc(outbufferlength*sizeof(float));
+  if (!outbuffer){
+    free(inbuffer);
+    return NULL;
+  }
+
+  int error;
+  SRC_STATE *src_state = src_new(SRC_LINEAR, 1, &error);
+  if (!src_state){
+    free(inbuffer);
+    free(outbuffer);
+    return NULL;
+  }
+
+  SRC_DATA src_data;
+  src_data.data_in = inbuffer;
+  src_data.data_out = outbuffer;
+  src_data.input_frames = inbufferlength;
+  src_data.output_frames = outbufferlength;
+  src_data.end_of_input = SF_TRUE;
+  src_data.src_ratio = sr_ratio;
+
+  /* sample rate conversion */ 
+  if (error = src_process(src_state, &src_data)){
+    free(inbuffer);
+    free(outbuffer);
+    src_delete(src_state);
+    return NULL;
+  }
+
+  buflen = src_data.output_frames;
+
+  src_delete(src_state);
+  free(inbuffer);
+
+  return outbuffer;
 } 
 
 
+float* ph_readaudio(const char *filename, int sr, int channels, float *sigbuf, int &buflen,\
+		    const float nbsecs){
+	if(!filename || sr <= 0)
+		return NULL;
+    return ph_readaudio2(filename, sr, sigbuf, buflen, nbsecs);
+}
+
 uint32_t* ph_audiohash(float *buf, int N, int sr, int &nb_frames){
 
    int frame_length = 4096;//2^12
@@ -308,7 +337,7 @@ uint32_t* ph_audiohash(float *buf, int N, int sr, int &nb_frames){
 
    //p = fftw_plan_dft_r2c_1d(frame_length,frame,pF,FFTW_ESTIMATE);
 
-   while (end <= N){
+   while (end < N){
        maxF = 0.0;
        maxB = 0.0;
        for (int i = 0;i<frame_length;i++){
@@ -365,6 +394,7 @@ uint32_t* ph_audiohash(float *buf, int N, int sr, int &nb_frames){
    return hash;
 }
 
+
 int ph_bitcount(uint32_t n){
     
     //parallel bit count
@@ -476,7 +506,7 @@ void *ph_audio_thread(void *p)
                 DP *dp = (DP *)s->hash_p[i];
                 int N, count;
 		pair<int,int> *p = (pair<int,int> *)s->hash_params;
-                float *buf = ph_readaudio(dp->id, p->first, p->second, N);
+                float *buf = ph_readaudio(dp->id, p->first, p->second, NULL, N);
                 uint32_t *hash = ph_audiohash(buf, N, p->first, count);
                 free(buf);
                 buf = NULL;
@@ -525,7 +555,7 @@ DP** ph_audio_hashes(char *files[], int count, int sr, int channels, int threads
                 s[n].hash_p = &hashes[start];
                 s[n].n = off;
 		s[n].hash_params = new pair<int,int>(sr,channels);
-                start = off;
+                start += off;
                 --rem;
                 pthread_create(&thds[n], NULL, ph_audio_thread, &s[n]);
         }
