@@ -34,10 +34,10 @@
 int ph_num_threads()
 {
 	int numCPU = 1;
-	#ifdef linux
+#ifdef __GLIBC__
 		numCPU = sysconf( _SC_NPROCESSORS_ONLN );
-	#else
-		nt mib[4];
+#else
+		int mib[2];
 		size_t len; 
 
 		mib[0] = CTL_HW;
@@ -56,7 +56,7 @@ int ph_num_threads()
      			}
 		}
 
-	#endif
+#endif
 	return numCPU;
 }
 #endif
@@ -459,7 +459,7 @@ DP** ph_dct_image_hashes(char *files[], int count, int threads)
                 s[n].hash_p = &hashes[start];
                 s[n].n = off;
                 s[n].hash_params = NULL;
-                start = off;
+                start += off;
                 --rem;
                 pthread_create(&thds[n], NULL, ph_image_thread, &s[n]);
         }
@@ -483,6 +483,7 @@ DP** ph_dct_image_hashes(char *files[], int count, int threads)
 CImgList<uint8_t>* ph_getKeyFramesFromVideo(const char *filename){
 
     long N =  GetNumberVideoFrames(filename);
+
     if (N < 0){
 	return NULL;
     }
@@ -756,7 +757,7 @@ DP** ph_dct_video_hashes(char *files[], int count, int threads)
                 s[n].hash_p = &hashes[start];
                 s[n].n = off;
                 s[n].hash_params = NULL;
-                start = off;
+                start += off;
                 --rem;
                 pthread_create(&thds[n], NULL, ph_video_thread, &s[n]);
         }
@@ -829,7 +830,7 @@ void ph_free_datapoint(DP *dp){
 }
 
 
-
+#ifdef HAVE_IMAGE_HASH
 DP** ph_read_imagehashes(const char *dirname,int pathlength, int &count){
 
     count = 0;
@@ -880,44 +881,6 @@ DP** ph_read_imagehashes(const char *dirname,int pathlength, int &count){
 
 }
 
-char** ph_readfilenames(const char *dirname,int &count){
-    count = 0;
-    struct dirent *dir_entry;
-    DIR *dir = opendir(dirname);
-    if (!dir)
-        return NULL;
-
-    /*count files */
-    while ((dir_entry = readdir(dir)) != NULL){
-	if (strcmp(dir_entry->d_name, ".") && strcmp(dir_entry->d_name,".."))
-	    count++;
-    }
-    
-    /* alloc list of files */
-    char **files = (char**)malloc(count*sizeof(*files));
-    if (!files)
-	return NULL;
-
-    errno = 0;
-    int index = 0;
-    char path[1024];
-    path[0] = '\0';
-    rewinddir(dir);
-    while ((dir_entry = readdir(dir)) != 0){
-	if (strcmp(dir_entry->d_name,".") && strcmp(dir_entry->d_name,"..")){
-	    strcat(path, dirname);
-	    strcat(path, "/");
-	    strcat(path, dir_entry->d_name);
-	    files[index++] = strdup(path);
-	}
-        path[0]='\0';
-    }
-    if (errno)
-	return NULL;
-    closedir(dir);
-    return files;
-}
-
 CImg<float>* GetMHKernel(float alpha, float level){
     int sigma = (int)4*pow((float)alpha,(float)level);
     static CImg<float> *pkernel = NULL;
@@ -933,6 +896,7 @@ CImg<float>* GetMHKernel(float alpha, float level){
     }
     return pkernel;
 }
+
 uint8_t* ph_mh_imagehash(const char *filename, int &N,float alpha, float lvl){
     if (filename == NULL){
 	return NULL;
@@ -943,7 +907,7 @@ uint8_t* ph_mh_imagehash(const char *filename, int &N,float alpha, float lvl){
     CImg<uint8_t> src(filename);
     CImg<uint8_t> img;
 
-    if (img.spectrum() == 3){
+    if (src.spectrum() == 3){
 	img = src.get_RGBtoYCbCr().channel(0).blur(1.0).resize(512,512,1,1,5).get_equalize(256);
     } else{
 	img = src.channel(0).get_blur(1.0).resize(512,512,1,1,5).get_equalize(256);
@@ -989,6 +953,45 @@ uint8_t* ph_mh_imagehash(const char *filename, int &N,float alpha, float lvl){
 
     return hash;
 }
+#endif
+
+char** ph_readfilenames(const char *dirname,int &count){
+    count = 0;
+    struct dirent *dir_entry;
+    DIR *dir = opendir(dirname);
+    if (!dir)
+        return NULL;
+
+    /*count files */
+    while ((dir_entry = readdir(dir)) != NULL){
+	if (strcmp(dir_entry->d_name, ".") && strcmp(dir_entry->d_name,".."))
+	    count++;
+    }
+    
+    /* alloc list of files */
+    char **files = (char**)malloc(count*sizeof(*files));
+    if (!files)
+	return NULL;
+
+    errno = 0;
+    int index = 0;
+    char path[1024];
+    path[0] = '\0';
+    rewinddir(dir);
+    while ((dir_entry = readdir(dir)) != 0){
+	if (strcmp(dir_entry->d_name,".") && strcmp(dir_entry->d_name,"..")){
+	    strcat(path, dirname);
+	    strcat(path, "/");
+	    strcat(path, dir_entry->d_name);
+	    files[index++] = strdup(path);
+	}
+        path[0]='\0';
+    }
+    if (errno)
+	return NULL;
+    closedir(dir);
+    return files;
+}
 
 
 int ph_bitcount8(uint8_t val){
@@ -1091,14 +1094,14 @@ DP* ph_read_datapoint(MVPFile *m){
     return dp;
 }
 
-void ph_mvp_init(MVPFile *m)
+__attribute__((__deprecated__)) void ph_mvp_init(MVPFile *m)
 {
 	if(!m) return;
 	
 	m->branchfactor = 2;
     	m->pathlength = 5;
     	m->leafcapacity = 23;
-    	#ifdef linux
+    	#ifdef __GLIBC__
 		m->pgsize = sysconf(_SC_PAGE_SIZE);
 	#else
 		m->pgsize = getpagesize();
@@ -1524,7 +1527,7 @@ querycleanup:
 }
 
 
-MVPRetCode ph_query_mvptree(MVPFile *m, DP *query, int knearest, float radius, float threshold,
+__attribute__((__deprecated__)) MVPRetCode ph_query_mvptree(MVPFile *m, DP *query, int knearest, float radius, float threshold,
                                                DP **results, int &nbfound){
     /*use host pg size until file pg size used can be determined  */
     m->pgsize = sysconf(_SC_PAGESIZE);
@@ -2065,7 +2068,7 @@ cleanup:
 }
 
 
-MVPRetCode ph_save_mvptree(MVPFile *m, DP **points, int nbpoints){
+__attribute__((__deprecated__)) MVPRetCode ph_save_mvptree(MVPFile *m, DP **points, int nbpoints){
 
     if (m->pgsize == 0) /*use host pg size as default */
 	m->pgsize = sysconf(_SC_PAGE_SIZE);
@@ -2509,7 +2512,7 @@ addcleanup:
 
 
 
-MVPRetCode ph_add_mvptree(MVPFile *m, DP **points, int nbpoints, int &nbsaved){
+__attribute__((__deprecated__)) MVPRetCode ph_add_mvptree(MVPFile *m, DP **points, int nbpoints, int &nbsaved){
     nbsaved = 0;
     /* open main file */
     char mainfile[256];
@@ -2751,33 +2754,3 @@ TxtMatch* ph_compare_text_hashes(TxtHashPoint *hash1, int N1, TxtHashPoint *hash
     return found_matches;
 }
 
-static bool keepStats = false;
-struct ph_stats
-{
-	ulong64 reads;
-	ulong64 avg_atime, height;
-	
-};
-enum ph_option
-{
-	PH_STATS
-};
-
-const ph_stats ph_get_stats(MVPFile *m)
-{
-	if(keepStats)
-	{
-	
-	}
-}
-void ph_set_option(ph_option opt, int val)
-{
-	switch(opt)
-	{
-		case PH_STATS:
-			keepStats = (bool)val;
-			break;
-		default:
-			break;
-	}
-}
